@@ -1,9 +1,9 @@
-from collections import defaultdict
-
 import logging
+from collections import defaultdict
+from typing import Union, Optional, Callable, DefaultDict, List, Dict, TypeVar, Type, Generic
+
 import zmq
 from pydantic import ValidationError, BaseModel
-from typing import Union, Optional, Callable, DefaultDict, List, Dict, TypeVar, Type, Generic
 
 from openmodule.models.base import ZMQMessage
 from openmodule.utils.schema import Schema
@@ -64,17 +64,47 @@ class MessageDispatcher:
         self.raise_validation_errors = raise_validation_errors
         self.raise_handler_errors = raise_handler_errors
 
+    def unregister_handler(self, listener: Listener):
+        for topic, listeners in self.listeners.items():
+            try:
+                listeners.remove(listener)
+            except ValueError:
+                pass
+
     def register_handler(self, topic: Union[bytes, str],
                          message_class: Type[ZMQMessageSub],
                          handler: Callable[[ZMQMessageSub], None], *,
                          filter: Optional[Dict] = None,
+                         match_type=False,
                          register_schema=True):
+        """
+        registers a message handler. without any filters all messages from the topic are
+        sent to the message handler.
+        :param filter: a dictionary of values which must match in order for the message to
+                       be further processed
+        :param match_type: if set to true the message_class's type field is used as a filter.
+                           equivalent to setting filter={"type": message_class.fields["type"].default}
+        """
+
         if hasattr(topic, "encode"):
             topic = topic.encode()
-        self.listeners[topic].append(Listener(message_class, filter, handler))
+
+        if match_type:
+            assert "type" in message_class.__fields__ and message_class.__fields__["type"].default, (
+                "Your message class definition does not set a `type` field, or the type field "
+                "does not have a default value!"
+            )
+            filter = filter or {}
+            filter["type"] = message_class.__fields__["type"].default
+
+        listener = Listener(message_class, filter, handler)
+        self.listeners[topic].append(listener)
+
         if register_schema:
             Schema.save_message(topic, message_class, handler, filter)
 
+        return listener
+
     def dispatch(self, topic: bytes, message: Union[Dict, BaseModel]):
         if isinstance(message, BaseModel):
             message = message.dict()
@@ -106,17 +136,48 @@ class MessageDispatcher:
 
 class SubscribingMessageDispatcher(MessageDispatcher):
     def __init__(self, subscribe: Callable[[bytes], None], name=None, *, raise_validation_errors=False,
-                 raise_handler_errors=False):
+                 raise_handler_errors=False, unsubscribe: Optional[Callable[[bytes], None]] = None):
         super().__init__(name=name, raise_validation_errors=raise_validation_errors,
                          raise_handler_errors=raise_handler_errors)
         self.subscribe = subscribe
+        self.unsubscribe = unsubscribe
 
     def register_handler(self, topic: Union[bytes, str],
                          message_class: Type[ZMQMessageSub],
                          handler: Callable[[ZMQMessageSub], None], *,
                          filter: Optional[Dict] = None,
-                         register_schema=True):
+                         register_schema=True,
+                         match_type=False):
         if topic and hasattr(topic, "encode"):
             topic = topic.encode()
         self.subscribe(topic)
-        return super().register_handler(topic, message_class, handler, filter=filter, register_schema=register_schema)
+        return super().register_handler(topic, message_class, handler, filter=filter,
+                                        register_schema=register_schema, match_type=match_type)
+
+    def unregister_handler(self, listener: Listener):
+        super().unregister_handler(listener)
+
+        warn_no_unsubscribe = False
+        for topic, listeners in self.listeners.items():
+            if not listeners:
+                if self.unsubscribe:
+                    self.unsubscribe(topic)
+                else:
+                    warn_no_unsubscribe = True
+                    break
+
+        if warn_no_unsubscribe:
+            self.log.warning("All handlers were unregistered from a topic, but no unsubscribe method was configured. "
+                             "This may cause a performance overhead if too many unused subscriptions are kept. "
+                             "Consider passing a unsubscribe method.")
+
+
+class ZMQMessageDispatcher(SubscribingMessageDispatcher):
+    def __init__(self, sub_socket: zmq.Socket, name=None, *, raise_validation_errors=False, raise_handler_errors=False):
+        super().__init__(
+            subscribe=lambda x: sub_socket.subscribe(x),
+            unsubscribe=lambda x: sub_socket.unsubscribe(x),
+            name=name,
+            raise_validation_errors=raise_validation_errors,
+            raise_handler_errors=raise_handler_errors
+        )
