@@ -7,6 +7,7 @@ import zmq
 from pydantic import ValidationError, BaseModel
 from pydantic.main import ROOT_KEY
 
+from openmodule.config import settings
 from openmodule.messaging import get_sub_socket, get_pub_socket, receive_message_from_socket
 from openmodule.models.rpc import RPCResponse, RPCRequest
 from openmodule.rpc.common import channel_to_request_topic, channel_to_response_topic
@@ -135,7 +136,7 @@ class RPCServer(object):
                            "my resource: %s, resource specified in rpc: %s", self.resource, message.resource)
             return None
 
-        if self.log.level <= logging.DEBUG:
+        if settings.LOG_LEVEL == logging.DEBUG:
             self.log.debug("received RPC channel: %s, type: %s, request: %s", channel, message.type, message.request)
         else:
             self.log.info("received RPC channel: %s, type: %s", channel, message.type)
