@@ -4,7 +4,8 @@ from typing import List
 from openmodule.core import OpenModuleCore
 from openmodule.models.base import ZMQMessage
 from openmodule.rpc.server import RPCServer
-from openmodule.models.backend import AccessRequest, AccessResponse, CountMessage, MediumAccesses, Access
+from openmodule.models.backend import AccessRequest, AccessResponse, CountMessage, MediumAccesses, Access, \
+    BackendRegisterMessage, BackendRegisterRequestMessage
 
 
 class Backend:
@@ -24,7 +25,9 @@ class Backend:
 
         self.register_at_controller()
         self.core.messages.register_handler(b"count", CountMessage, self.handle_message)
-        self.core.messages.register_handler(b"backend", ZMQMessage, self.handle_backend_message)
+        self.core.messages.register_handler(b"backend", BackendRegisterRequestMessage,
+                                            self.handle_backend_register_request,
+                                            match_type=True)
 
     def register_rpcs(self, rpc_server: RPCServer):
         rpc_server.add_filter(self._backend_filter, "backend", "auth")
@@ -104,13 +107,11 @@ class Backend:
             return AccessResponse(success=True,
                                   medium=MediumAccesses(accesses=[], id=request.id, type=request.medium_type))
 
-    def handle_backend_message(self, message: ZMQMessage):
+    def handle_backend_register_request(self, _):
         """
         Registers the backend if the message type is register_request
         """
-        if message.type.lower() == "register_request":
-            self.register_at_controller()
+        self.register_at_controller()
 
     def register_at_controller(self):
-        message = ZMQMessage(name=self.core.config.NAME, type="register")
-        self.core.publish(message, b"backend")
+        self.core.publish(BackendRegisterMessage(), b"backend")
