@@ -3,8 +3,8 @@ from typing import Any
 from unittest import TestCase
 
 from openmodule.config import settings
-from openmodule.dispatcher import MessageDispatcher
-from openmodule.models.base import ZMQMessage
+from openmodule.dispatcher import MessageDispatcher, SubscribingMessageDispatcher
+from openmodule.models.base import ZMQMessage, OpenModuleModel
 
 
 class MessageDispatcherBaseTest(TestCase):
@@ -85,6 +85,33 @@ class MessageDispatcherBasicsTestCase(MessageDispatcherBaseTest):
         self.dispatcher.dispatch(b"test", self.dummy_message())
         self.assertIsNotNone(self.message)
 
+    def test_match_type(self):
+        class TypedZMQMessage(ZMQMessage):
+            type: str = "test1"
+
+        self.dispatcher.register_handler(b"test", TypedZMQMessage, self._set_true_handler, match_type=True)
+
+        # wrong type does not match
+        self.dispatcher.dispatch(b"test", ZMQMessage(type="test2"))
+        self.assertIsNone(self.message)
+
+        # correct type, message is dispatched
+        self.dispatcher.dispatch(b"test", ZMQMessage(type="test1"))
+        self.assertIsNotNone(self.message)
+
+    def test_match_type_without_default_value(self):
+        # empty string is also "false" thus not a valid type
+        with self.assertRaises(AssertionError) as e:
+            self.dispatcher.register_handler(b"test", ZMQMessage, self._set_true_handler, match_type=True)
+        self.assertIn("class definition does not set a `type` field", str(e.exception))
+
+        class MessageWithoutType(OpenModuleModel):
+            pass
+
+        with self.assertRaises(AssertionError) as e:
+            self.dispatcher.register_handler(b"test", MessageWithoutType, self._set_true_handler, match_type=True)
+        self.assertIn("class definition does not set a `type` field", str(e.exception))
+
 
 class MessageDispatcherWithoutExecutorTestCase(MessageDispatcherBaseTest):
     def test_exception_in_handler(self):
@@ -98,10 +125,46 @@ class MessageDispatcherWithoutExecutorTestCase(MessageDispatcherBaseTest):
         self.assertIn("something broke!", cm.output[0])
 
     def test_validation_error(self):
-        def handler(x): None
+        def handler(x):
+            pass
 
         handler.register_schema = False
         self.dispatcher.register_handler("test", ZMQMessage, handler, register_schema=False)
         with self.assertLogs() as cm:
             self.dispatcher.dispatch(b"test", {})
         self.assertIn("validation error", cm.output[0])
+
+
+class SubscribingMessageDispatcherTestCase(TestCase):
+    subscriptions = set()
+
+    def subscribe(self, topic):
+        self.subscriptions.add(topic)
+
+    def unsubscribe(self, topic):
+        self.subscriptions.remove(topic)
+
+    def setUp(self) -> None:
+        self.subscriptions.clear()
+        self.dispatcher = SubscribingMessageDispatcher(
+            subscribe=self.subscribe,
+            unsubscribe=self.unsubscribe
+        )
+
+    def test_subscribe(self):
+        self.dispatcher.register_handler(b"topic1", ZMQMessage, lambda *x: x, register_schema=False)
+        self.assertEqual({b"topic1"}, self.subscriptions)
+
+    def test_unsubscribe(self):
+        listener1 = self.dispatcher.register_handler(b"topic1", ZMQMessage, lambda *x: x, register_schema=False)
+        listener2 = self.dispatcher.register_handler(b"topic1", ZMQMessage, lambda *x: x, register_schema=False)
+
+        self.assertEqual({b"topic1"}, self.subscriptions)
+
+        # listener 2 is still subscribed, topic should still be listed in subscriptions
+        self.dispatcher.unregister_handler(listener1)
+        self.assertEqual({b"topic1"}, self.subscriptions)
+
+        # now no listener is subscribed anymore
+        self.dispatcher.unregister_handler(listener2)
+        self.assertEqual(set(), self.subscriptions)
