@@ -1,6 +1,6 @@
 from unittest import TestCase
 
-from openmodule.core import OpenModuleCore
+from openmodule.core import OpenModuleCore, init_openmodule, shutdown_openmodule
 from openmodule.health import health_ok, health_error
 from openmodule_test.health import HealthTestMixin
 
@@ -10,13 +10,12 @@ class HealthTestCase(HealthTestMixin, TestCase):
 
     def setUp(self):
         super().setUp()
-        self.core = OpenModuleCore(self.zmq_context(), self.zmq_config())
-        self.core.start()
+        self.core = init_openmodule(context=self.zmq_context(), config=self.zmq_config())
+        self.addCleanup(shutdown_openmodule)
         self.wait_for_health()
 
     def tearDown(self):
         super().tearDown()
-        self.core.join()
 
     def test_health_ok(self):
         self.core.health.health_handler = lambda: health_ok({"some": "meta"})
