@@ -1,13 +1,14 @@
 import logging
 import time
 from functools import partial
+from typing import Union
 from unittest import TestCase
 from uuid import uuid4
 
 from pydantic.types import conint
 
 from openmodule.config import settings, override_context
-from openmodule.models.base import OpenModuleModel
+from openmodule.models.base import OpenModuleModel, EmptyModel, ZMQMessage
 from openmodule.rpc import server
 from openmodule.rpc.client import RPCClient
 from openmodule.rpc.common import channel_to_response_topic, channel_to_request_topic
@@ -87,6 +88,7 @@ class TestRPCResponse2(OpenModuleModel):
 
 class RPCServerTestCase(RPCServerTestMixin):
     rpc_channels = ["channel", "channel2"]
+    topics = ["test"]
 
     server: RPCServer
 
@@ -368,6 +370,99 @@ class RPCServerTestCase(RPCServerTestMixin):
         with self.assertRaises(TimeoutError):
             self.rpc("channel", "test-type", {}, resource="test-2")
 
+    def test_union_response_parsing(self):
+        """
+        tests that the rpc server correctly response when Union types are used as responses
+        """
+
+        class Integer(OpenModuleModel):
+            some_int: int
+
+        class String(OpenModuleModel):
+            some_str: str
+
+        def dict_handler(*_, **__):
+            """Test handler."""
+            return {"some_str": "str"}
+
+        def integer_handler(*_, **__):
+            """Test handler."""
+            return Integer(some_int=1)
+
+        def string_handler(*_, **__):
+            """Test handler."""
+            return String(some_str=1)
+
+        def none_handler(*_, **__):
+            """Test handler."""
+            return
+
+        def test_response_handler(*_, **__):
+            """Test handler."""
+            return TestRPCResponse()
+
+        class response_type(OpenModuleModel):
+            __root__: Union[Integer, String]
+
+        self.server.register_handler("channel", "dict", EmptyModel, response_type, dict_handler)
+        self.server.register_handler("channel", "string", EmptyModel, response_type, string_handler)
+        self.server.register_handler("channel", "integer", EmptyModel, response_type, integer_handler)
+        self.server.register_handler("channel", "none", EmptyModel, response_type, none_handler)
+        self.server.register_handler("channel", "test_response", EmptyModel, response_type, test_response_handler)
+        self.wait_for_rpc_server(self.server)
+
+        # dict, should work
+        response = self.rpc("channel", "dict", {}, resource=None)
+        self.assertRPCSuccess(response)
+        self.assertEqual(response["response"], {"some_str": "str", "status": "ok"})
+
+        # string response, should work
+        response = self.rpc("channel", "string", {}, resource=None)
+        self.assertRPCSuccess(response)
+        self.assertEqual(response["response"], {"some_str": "1", "status": "ok"})
+
+        # integer response, should work
+        response = self.rpc("channel", "integer", {}, resource=None)
+        self.assertRPCSuccess(response)
+        self.assertEqual(response["response"], {"some_int": 1, "status": "ok"})
+
+        # none response, should not work, as required fields are not set
+        response = self.rpc("channel", "none", {}, resource=None)
+        self.assertGreater(response["response"].items(), {"status": "handler_error"}.items())
+
+        # test_response, an incorrect response should not work
+        response = self.rpc("channel", "none", {}, resource=None)
+        self.assertGreater(response["response"].items(), {"status": "handler_error"}.items())
+
+    def test_async_receive(self):
+        """
+        this test cases shows a usage example for async receive. The handler emits a message before the
+        rpc response. in this case we need to first capture the response. This is a limitation because
+        the rpc mixin uses the same zmq client as the other wait for message functions
+        """
+
+        def publishing_handler(*_, **__):
+            """test handler which publishes a message"""
+            message = ZMQMessage(type="abc")
+            message.publish_on_topic(self.server.pub, b"test")
+
+        self.server.register_handler("channel", "test", EmptyModel, EmptyModel, publishing_handler)
+        self.wait_for_rpc_server(self.server)
+
+        # this shows the current limitation in the test framework, if we ever do change the setup to better
+        # support capturing messages from multiple topics, case will fail
+        response = self.rpc("channel", "test", EmptyModel())
+        self.assertRPCSuccess(response)
+        with self.assertRaises(TimeoutError):
+            self.zmq_client.wait_for_message_on_topic(b"test")
+
+        # this shows how we can capture the message
+        rpc_id = self.rpc("channel", "test", EmptyModel(), receive_response=False)
+        message = self.zmq_client.wait_for_message_on_topic(b"test")
+        self.assertEqual(message["type"], "abc")
+        response = self.receive_rpc_response("channel", rpc_id)
+        self.assertRPCSuccess(response)
+
 
 class RpcClientTest(RPCServerTestMixin, OpenModuleCoreTestMixin):
     rpc_channels = ["channel"]
@@ -413,6 +508,7 @@ class RpcClientTest(RPCServerTestMixin, OpenModuleCoreTestMixin):
         self.assertEqual("ok", result.status)
         self.assertIn("some_payload", result.response)
 
+
 class RpcClientResourceTest(RPCServerTestMixin, OpenModuleCoreTestMixin):
     rpc_channels = ["channel"]
 
@@ -420,7 +516,8 @@ class RpcClientResourceTest(RPCServerTestMixin, OpenModuleCoreTestMixin):
         super().setUp()
         self.server = RPCServer(config=self.zmq_config(), context=self.zmq_context(), filter_resource=True)
         self.server.register_handler("channel", "test", TestRPCRequest, TestRPCResponse2, RpcClientTest.handler1)
-        self.server.register_handler("channel", "timeout", TestRPCRequest, TestRPCResponse2, RpcClientTest.timeout_handler)
+        self.server.register_handler("channel", "timeout", TestRPCRequest, TestRPCResponse2,
+                                     RpcClientTest.timeout_handler)
         self.server.run_as_thread()
         self.wait_for_rpc_server(self.server)
 
@@ -433,7 +530,7 @@ class RpcClientResourceTest(RPCServerTestMixin, OpenModuleCoreTestMixin):
 
         with override_context(RESOURCE="asdf"):
             with self.assertRaises(TimeoutError):
-                result = self.client.rpc(b"channel", "test", {}, timeout=1)
+                self.client.rpc(b"channel", "test", {}, timeout=1)
 
         with override_context(RESOURCE=""):
             result = self.client.rpc(b"channel", "test", {}, timeout=1)
